# demo使用教程

## 1、code generation：

    进入resources目录并执行：
    java -jar avro-tools-1.8.2.jar compile -string schema Record.avsc [代码生成路径]
    demo的目录中已经生成了代码，可以跳过此步骤

## 2、用maven打包：

    进入pom文件所在的目录并执行：
    mvn clean assembly:assembly -Dmaven.test.skip=true
    或者直接用idea打包

## 3、提交至flink，启动消费：

    ./bin/flink run ~/consumerDemo-avro-flink-1.0-SNAPSHOT.jar --brokers xxx --topic xxx --group xxx --user xxx --password xxx —trans2sql

# 文件说明

    resources/Record.avsc是定义的协议文件，共定义了14个schema：
        Field是描述数据库中列的元数据；
        Integer、Character等是描述列值的元数据；
        Record才是从kafka中消费的数据格式。
    resources/avro-tools-1.8.2.jar是用来生成代码的工具包
    java/com/tencent/subscribe/avro路径下存放的是已经生成的代码。

# 启动参数说明

| 参数      |       说明                      |
| ----------| -------------------------------|
| brokers   | Kafka 的内网访问地址（域名+端口号） |
| topic     | Kafka topic 名称                |
| group     | Kafka 消费组名称                 |
| user      | Kafka 消费组账号名               |
| password  | Kafka 消费组密码                 |
| trans2sql | 是否将消费数据转换为 SQL 语句打印   |

# 协议文件

## Record中字段说明

| 字段名称               |含义                    | 备注       |
| ---------------       | ----------            | ----------|
| id                    | 全局递增Id             |                 |
| version               | 协议版本               |                |
| messageType           | 消息类型               |                |
| fileName              | binlog文件名           |               |
| position              | binlog位点             |               |
| safePosition          | 事务开始的binlog位点    |                |
| timestamp             | binlog中的时间戳        |              |
| gtid                  | 事务gtid               |              |
| transactionId         | 事务Id                 |           |
| serverId              | 源库serverId           |            |
| threadId              | 源库线程ID              |            |
| sourceType            | 源库类型                |            |
| sourceVersion         | 源库版本                | 等价于 select version();   |
| schemaName            | 库名                   |            |
| tableName             | 表名                   |            |
| objectName            | 库名.表名               |              |
| columns               | 列元数据                |                |
| oldColumns            | dml前的列值             |               |
| newColumns            | dml后的列值             |               |
| sql                   | ddl的sql语句            |                |
| executionTime         | ddl执行时长              |                 |
| heartbeatTimestamp    | 心跳消息的时间戳          | 只有heartbeat事件有意义   |
| syncedGtid            | 已解析gtid集合          |                 | 
| fakeGtid              | 是否为构造的假gtid       |                |
| pkNames               | 主键字段                 | 只有dml可能有值  | 
| readerTimestamp       | 后台处理这条数据的时间    |  毫秒数         |
| tags                  | 一些附加的字段           |               |
| tags.lowerCaseTableNames| 表名大小写敏感         |                |
| total                 | 无                     | 如果消息分片，记录分片总数。当前版本(version=1)无意义，预留扩展 |
| index                 | 无                     | 如果消息分片，记录分片总数。当前版本(version=1)无意义，预留扩展 |

## mysql列属性

Field是描述列属性的对象。包含四个属性：

+ name 列名
+ dataTypeNumber 是binlog中记录的数据类型。取值详见：https://dev.mysql.com/doc/internals/en/com-query-response.html
+ isKey 是否主键
+ originalType ddl中定义的类型

## mysql数据类型转换逻辑

在avro协议中，将mysql类型转换成了以下11中格式。

### Integer

对应数据库中的整数类型，如：TINYINT SMALLINT MEDIUMINT INT BIGINT 和 BIT。  
拥有两个属性：precision表示整数占有的字节数；value为整数值。对于BIT类型，precision固定为8。

### Character

对应数据库中的字符类型，如：CHAR VARCHAR TEXT等。

### Float

对应数据库中的浮点类型，如：FLOAT 和 DOUBLE。 precision和scale分别为information_schema.COLUMNS表中的数据。    
注意，如果表或者列是在订阅任务启动之后新增的，则无法获取到precision和scale。

### Decimal

对应数据库中DECIMAL。

### Timestamp

对应数据库中的TIMESTAMP。其中timestamp单位为秒，millis单位为毫秒，范围为[0,999]。

### DateTime

对应数据库中的其他时间类型

### TimestampWithTimeZone

暂未使用

### BinaryGeometry

对应数据库中的geometry及其子类。

### TextGeometry

暂未使用

### BinaryObject

对应数据库中的二进制类型，如BINARY VARBINARY 和 BLOB等。

### TextObject

对应数据库中的文本类型，如ENUM SET 和 JSON。

### EmptyObject

对应NULL

# 注意

    trans2sql时，使用了 javax.xml.bind.DatatypeConverter.printHexBinary() 将byte[]转成16进制。
    如果不需要转sql，可以注释此处代码。如果需要请使用jdk1.8避免不兼容。