package com.tencent.subscribe.serialize;

import com.tencent.subscribe.beans.RecordMsgObject;
import org.apache.flink.api.common.typeinfo.TypeInformation;
import org.apache.flink.streaming.connectors.kafka.KafkaDeserializationSchema;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.common.header.Header;
import java.util.Iterator;

public class DeserializeProtobufToRecordMsgObject implements KafkaDeserializationSchema<RecordMsgObject> {


    private final static String partitionSeqKey = "ps";

    @Override
    public boolean isEndOfStream(RecordMsgObject obj) {
        return false;
    }

    // 从kafka中收到一个envlope
    @Override
    public RecordMsgObject deserialize(ConsumerRecord<byte[], byte[]> record) throws Exception {
        RecordMsgObject obj = new RecordMsgObject();
        obj.topic = record.topic();
        obj.partition = record.partition();
        obj.offset = record.offset();
        obj.partitionSeq = getPartitionSeq(record);
        obj.key = new String(record.key());
        obj.headers = record.headers();
        // 这里收到的就是Envelope的二进制的值
        obj.value = record.value();
        return obj;
    }

    // 输入的数据类型
    @Override
    public TypeInformation<RecordMsgObject> getProducedType() {
        return TypeInformation.of(RecordMsgObject.class);
    }

    private static long getPartitionSeq(ConsumerRecord<byte[], byte[]> msg) {
        Iterator<Header> it = msg.headers().headers(partitionSeqKey).iterator();
        if (it.hasNext()) {
            Header h = it.next();
            return Long.parseUnsignedLong(new String(h.value()));
        }

        throw new IllegalStateException(partitionSeqKey + " does not exists");
    }
}